function [paraest,t_sta,V,it,Chi_sta,Pvalue]=gmmest(moment,para0,Y,X,Z,n_it,K)
%This program is for  GMM estimation. 
%input:
%moment: moment conditions function defined by users
%para0:initial value for estimated parameters
%Y,X:data used to estimate parameters
%Z: data for instrument variables
%n_it: maximum convergence number when choosing optimal weighting matrix
%K:number of moment conditions
%output:   
%paraest:parameters estimated
%t_sta: T statistics for each estimated parameter
%V:covariance matrix for estimated parameters
%it: number of iteration
%Chi_sta and Pvalue: overidentifying test, null hypothesis is moment
%conditions are feasible
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Edited by Cao Zhiguang 
%E-mail:caozhiguang@21cn.com
%Date:2006/05/01
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%modified by Jess Ruiz (E-mail: jruizand@ucm.es; Date: 2020/09/05 
%%%%%%%%%%main code for GMM
nlag=round(length(Y)^(1/3));
W(:,:,1)=eye(K);
options=optimset('Display','off','MaxFunEvals',100000,...
    'MaxIter',100000,'TolFun',0.00001);
[para(:,1),fv(:,1)]=fminsearch(moment,para0,options,1,Y,X,Z,W(:,:,1),1);
for i=2:n_it
    m_t=feval(moment,para(:,i-1),2,Y,X,Z,W(:,:,i-1),1);
    W(:,:,i)=gmmweightmatrix(m_t,nlag);
    [para(:,i),fv(:,i)]=fminsearch(moment,para(:,i-1),options,1,Y,X,Z,W(:,:,i),1);
    if abs(fv(:,i)-fv(:,i-1))/abs(fv(:,i-1))<1e-4||fv(i)<=1e-15
        break
    end
end 
it=i;
paraest1=exp(para(1,it))/(1+exp(para(1,it)));
if it==n_it
    error('number of iteration exceeds defined maximium number')
else 
    paraest=[paraest1;para(2,it)];
    m0=feval(moment,paraest,3,Y,X,Z,W(:,:,it),0);
    for j=1:length(para0)
        a=zeros(length(para0),1);
        eps=max(paraest(j)*1e-4,1e-5);
        a(j)=eps;
        m1=feval(moment,paraest+a,3,Y,X,Z,W(:,:,it),0);
        M(:,j)=(m1-m0)/eps;
    end
end
V=pinv(M'*W(:,:,it)*M)/size(Y,1);  
stderror=sqrt(diag(V));
t_sta=paraest./(stderror);
Chi_sta=size(Y,1)*(fv(it));
Pvalue=1-chi2cdf(Chi_sta,K-length(para0));
