% Edited by Jess Ruiz (jruizand@ucm.es)
clear
clc
close all
%
global Yt Ut
load gdp_u_us.prn -ascii
gdp_u=gdp_u_us;
Yt=log(gdp_u(1:end,1));
cyt=Yt-hpjra(length(Yt),1600)*Yt;
Ut=gdp_u(1:end,2)./100;
T=length(Yt);
%transformed parameters:
param0tr=[2.351464 2.355202 3.056895  2.693567  5.635313 -0.335523 -0.175425   0    5.350932  4.328002]';
        %[phi_tr    phi2_tr sigmav_tr sigmae_tr sigmaw_tr alpha0     alpha1  alpha2  sigmaec   sigmavl]';
% Maximum likelihood Estimation 
options=optimset('Display','Iter','MaxFunEvals',10000,'MaxIter',10000,... 
    'TolFun',0.00001);
param1tr=fminsearch('kf_detrend',param0tr,options,0); 
param0tr=param1tr; 
param1tr=fminunc('kf_detrend',param0tr,options,0);
%
% Estimated parameters
sigmavopt=exp(-param1tr(3))/10;
sigmaeopt=exp(-param1tr(4))/10;
sigmawopt=exp(-param1tr(5))/10;
d1=param1tr(1)/(1+abs(param1tr(1)));
d2=param1tr(2)/(1+abs(param1tr(2)));
phiopt=d1+d2;
phi2opt=-d1*d2;
sigmaecopt=exp(-param1tr(9))/10;
sigmavlopt=exp(-param1tr(10))/10;
alpha0opt=param1tr(6);
alpha1opt=param1tr(7);
alpha2opt=param1tr(8);
%
%Vector of optimal parameters
paramopt=[phiopt phi2opt sigmavopt sigmaeopt sigmawopt alpha0opt alpha1opt alpha2opt sigmaecopt sigmavlopt]';
%
[lfv,Xt1t,xtt,Ptt,xtT] = kf_detrend(paramopt,1);
%
outputgap=100.*xtt(2,26:end); %%%%%%%%% output-gap (%)
%Computing stantadard deviations of trend-component ofunemployment rate
dtxtt=zeros(1,length(xtt));
    for i=1:length(xtt)
        dtxtt(i)=Ptt(6,6*i)^0.5;
    end
bsup=xtt(6,:)+1.645.*dtxtt;
binf=xtt(6,:)-1.645.*dtxtt;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Computing stantadard deviations of trend-component of output
dtxtty=zeros(1,length(xtt));
    for i=1:length(xtt)
        dtxtty(i)=Ptt(2,2*i)^0.5;
    end
bygapsup=xtt(2,:)+1.645.*dtxtty;
bygapinf=xtt(2,:)-1.645.*dtxtty;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
figure;
set(gca,'fontsize',18)
startDate=datenum('06-30-1954'); 
endDate=datenum('03-31-2018');
xData=linspace(startDate,endDate,256);
dateFormat=17;
yy=[100.*bsup(26:end)' 100.*binf(26:end)'-100.*bsup(26:end)'];
ha=area(xData,yy,'FaceColor',[0.9 0.9 0.9]);
set(gca,'XTick',linspace(xData(1), xData(end),15));
datetick('x','QQ-yy','keepticks');
xlim([xData(1) xData(end)]);
ylim([0 12]);
set(ha(1),'FaceColor','none');
set(ha, 'LineStyle', 'none');
abcisa=1:1:length(bsup(26:end));
hold on 
hh1=area([xData(1) xData(1)], [12 12],'FaceColor',[0.8 0.85 0.85],'FaceAlpha',0.5);
set(hh1, 'LineStyle', 'none');
hh2=area([xData(14) xData(17)], [12 12],'FaceColor',[0.8 0.85 0.85],'FaceAlpha',0.5);
set(hh2, 'LineStyle', 'none');
hh3=area([xData(25) xData(28)], [12 12],'FaceColor',[0.8 0.85 0.85],'FaceAlpha',0.5);
set(hh3, 'LineStyle', 'none');
hh4=area([xData(63) xData(67)], [12 12],'FaceColor',[0.8 0.85 0.85],'FaceAlpha',0.5);
set(hh4, 'LineStyle', 'none');
hh5=area([xData(79) xData(84)], [12 12],'FaceColor',[0.8 0.85 0.85],'FaceAlpha',0.5);
set(hh5, 'LineStyle', 'none');
hh6=area([xData(104) xData(106)], [12 12],'FaceColor',[0.8 0.85 0.85],'FaceAlpha',0.5);
set(hh6, 'LineStyle', 'none');
hh7=area([xData(110) xData(115)], [12 12],'FaceColor',[0.8 0.85 0.85],'FaceAlpha',0.5);
set(hh7, 'LineStyle', 'none');
hh8=area([xData(146) xData(148)], [12 12],'FaceColor',[0.8 0.85 0.85],'FaceAlpha',0.5);
set(hh8, 'LineStyle', 'none');
hh9=area([xData(188) xData(191)], [12 12],'FaceColor',[0.8 0.85 0.85],'FaceAlpha',0.5);
set(hh9, 'LineStyle', 'none');
hh10=area([xData(215) xData(221)], [12 12],'FaceColor',[0.8 0.85 0.85],'FaceAlpha',0.5);
set(hh10, 'LineStyle', 'none');
hd=plot(xData,100.*Ut(26:end),'-.','LineWidth',2.5, 'Color','r'); %,'DisplayName','Tasa de Paro');
hf=plot(xData,100.*xtt(6,26:end)','LineWidth',1, 'Color','k'); %,'DisplayName','NAIRU');
title('\fontsize{20}Unemployment rate (%), Trend component (%) and confidence bands (90%)');
grid on
hold off
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
figure;
set(gca,'fontsize',14)
startDate=datenum('06-30-1954'); 
endDate=datenum('03-31-2018');
xData=linspace(startDate,endDate,256);
dateFormat=17;
set(gca,'XTick',linspace(xData(1), xData(end),20));
datetick('x','QQ-yy','keepticks');
xlim([xData(1) xData(end)]);
ylim([-7 7]);
abcisa=1:1:length(bygapsup(26:end));
hold on 
hh1=area([xData(1) xData(1)], [7 7],-7,'FaceColor',[0.8 0.85 0.85],'FaceAlpha',0.5);
set(hh1, 'LineStyle', 'none');
hh2=area([xData(14) xData(17)], [7 7],-7,'FaceColor',[0.8 0.85 0.85],'FaceAlpha',0.5);
set(hh2, 'LineStyle', 'none');
hh3=area([xData(25) xData(28)], [7 7],-7,'FaceColor',[0.8 0.85 0.85],'FaceAlpha',0.5);
set(hh3, 'LineStyle', 'none');
hh4=area([xData(63) xData(67)], [7 7],-7,'FaceColor',[0.8 0.85 0.85],'FaceAlpha',0.5);
set(hh4, 'LineStyle', 'none');
hh5=area([xData(79) xData(84)], [7 7],-7,'FaceColor',[0.8 0.85 0.85],'FaceAlpha',0.5);
set(hh5, 'LineStyle', 'none');
hh6=area([xData(104) xData(106)], [7 7],-7,'FaceColor',[0.8 0.85 0.85],'FaceAlpha',0.5);
set(hh6, 'LineStyle', 'none');
hh7=area([xData(110) xData(115)], [7 7],-7,'FaceColor',[0.8 0.85 0.85],'FaceAlpha',0.5);
set(hh7, 'LineStyle', 'none');
hh8=area([xData(146) xData(148)], [7 7],-7,'FaceColor',[0.8 0.85 0.85],'FaceAlpha',0.5);
set(hh8, 'LineStyle', 'none');
hh9=area([xData(188) xData(191)], [7 7],-7,'FaceColor',[0.8 0.85 0.85],'FaceAlpha',0.5);
set(hh9, 'LineStyle', 'none');
hh10=area([xData(215) xData(221)], [7 7],-7,'FaceColor',[0.8 0.85 0.85],'FaceAlpha',0.5);
set(hh10, 'LineStyle', 'none');
hf=plot(xData,100.*(Ut(26:end)-xtt(6,26:end)'),'LineWidth',3,'Color','k');   %,'LineWidth',1, 'Color','k'); %,'DisplayName','NAIRU');
title('\fontsize{20} Cyclical component of unemployment rate (%)');
grid on
hold off

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
figure;
set(gca,'fontsize',14)
startDate=datenum('06-30-1954'); 
endDate=datenum('03-31-2018');
xData=linspace(startDate,endDate,256);
dateFormat=17;
set(gca,'XTick',linspace(xData(1), xData(end),20));
datetick('x','QQ-yy','keepticks');
xlim([xData(1) xData(end)]);
ylim([-7 7]);
abcisa=1:1:length(bygapsup(26:end));
hold on 
hh1=area([xData(1) xData(1)], [7 7],-7,'FaceColor',[0.8 0.85 0.85],'FaceAlpha',0.5);
set(hh1, 'LineStyle', 'none');
hh2=area([xData(14) xData(17)], [7 7],-7,'FaceColor',[0.8 0.85 0.85],'FaceAlpha',0.5);
set(hh2, 'LineStyle', 'none');
hh3=area([xData(25) xData(28)], [7 7],-7,'FaceColor',[0.8 0.85 0.85],'FaceAlpha',0.5);
set(hh3, 'LineStyle', 'none');
hh4=area([xData(63) xData(67)], [7 7],-7,'FaceColor',[0.8 0.85 0.85],'FaceAlpha',0.5);
set(hh4, 'LineStyle', 'none');
hh5=area([xData(79) xData(84)], [7 7],-7,'FaceColor',[0.8 0.85 0.85],'FaceAlpha',0.5);
set(hh5, 'LineStyle', 'none');
hh6=area([xData(104) xData(106)], [7 7],-7,'FaceColor',[0.8 0.85 0.85],'FaceAlpha',0.5);
set(hh6, 'LineStyle', 'none');
hh7=area([xData(110) xData(115)], [7 7],-7,'FaceColor',[0.8 0.85 0.85],'FaceAlpha',0.5);
set(hh7, 'LineStyle', 'none');
hh8=area([xData(146) xData(148)], [7 7],-7,'FaceColor',[0.8 0.85 0.85],'FaceAlpha',0.5);
set(hh8, 'LineStyle', 'none');
hh9=area([xData(188) xData(191)], [7 7],-7,'FaceColor',[0.8 0.85 0.85],'FaceAlpha',0.5);
set(hh9, 'LineStyle', 'none');
hh10=area([xData(215) xData(221)], [7 7],-7,'FaceColor',[0.8 0.85 0.85],'FaceAlpha',0.5);
set(hh10, 'LineStyle', 'none');
hf=plot(xData,100.*xtt(2,26:end)','LineWidth',2, 'Color','k'); %,'DisplayName','NAIRU');
hd=plot(xData,100.*cyt(26:end),'-.','LineWidth',2.5, 'Color','r');
title('\fontsize{20}Cyclical component of GDP (%)');
grid on
hold off
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Computing Covariance Matrix of optimal parameters
%
pstar=paramopt;
n=length(pstar);
    Vepsi=ones(length(pstar),1).*1e-06;
    Mepsi=diag(Vepsi);
    for j=1:length(pstar)
        Mepsi(j,j)=max(pstar(j)*1e-04,1e-06);
    end
    eps=Mepsi; eps(6,6)=1e-05;
Hess=zeros(n,n);
for i=1:n
    for j=1:n
        Hess(i,j)=(kf_detrend(pstar+eps(:,i)+eps(:,j),1)-...
                  kf_detrend(pstar+eps(:,i),1)-kf_detrend(pstar+eps(:,j),1)+...
                  kf_detrend(pstar,1))/(eps(j,j)*eps(i,i));
    end
end
Hess=inv(Hess);
%[pstar sqrt(diag(Hess))]
sevec=sqrt(diag(Hess));
%
format long;
varnames={'phi_1';'phi_2';'sigma_v';'sigma_e';'sigma_w';'alpha_0';'alpha_1';'alpha_2';'sigma_ec';'sigma_vl'};
Coefficient=paramopt;
Std_Error=sevec;
t_statistic=Coefficient./Std_Error;
p_value=1-normcdf(abs(t_statistic),0,1);
disp('____________________________________________________');
disp('Estimations');
TABLA=table(Coefficient,Std_Error,p_value,'RowNames',varnames)
disp(['log(likelihood):   ' num2str(-lfv)])
disp('____________________________________________________');
%