function A=hp(T,M)

%HODRICK  The Hodrick-Prescott filter. For a given sample size T and a given
%         value for the multiplier M the function HODRICK(T,M) returns a
%         filter matrix so that if Y is a column vector containing the log of
%         some variable the trend is obtained by HODRICK(T,M)*Y. The value M
%         uses to be 1600 for quarterly data and 400 for annual data.

%         Routine: Matlab for Windows, ver.4.0

%         See E.C. Prescott (1986) Theory Ahead Business Cycle Measurement.
%         FRBM Quarterly Review.

  B(1,1) =  1+M       ;
  B(1,2) = -2*M       ;
  B(1,3) =  M         ;
  B(2,1) = -2*M       ;
  B(2,2) =  1+5*M     ;
  B(2,3) = -4*M       ;
  B(2,4) =  M         ;
  for t=3:T-2
    B(t,t-2) =  M     ;
    B(t,t-1) = -4*M   ;
    B(t,t)   =  1+6*M ;
    B(t,t+1) = -4*M   ;
    B(t,t+2) =  M     ;
  end
  B(T-1,T-3) =  M     ;
  B(T-1,T-2) = -4*M   ;
  B(T-1,T-1) =  1+5*M ;
  B(T-1,T)   = -2*M   ;
  B(T,T-2)   =  M     ;
  B(T,T-1)   = -2*M   ;
  B(T,T)     =  1+M   ;

  A = inv(B);

%  HP.M
%  END OF MATLAB FILE
